import {
  ConflictException,
  ForbiddenException,
  forwardRef,
  Inject,
  Injectable,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Equipo } from '../equipo/entity/equipo.entity';
import { EquipoPrograma } from './entity/equipo-programa.entity';
import { Programa } from '../institucion-programa/entity/programa.entity';
import { FullInformacionEquipoProgramaView } from './entity/views/full-informacion-equipo-programa.view';
import { InformacionEquipoProgramaView } from './entity/views/informacion-equipo-programa.view';
import { EquipoService } from '../equipo/equipo.service';
import { InstitucionProgramaService } from '../institucion-programa/institucion-programa.service';
import { Operador } from '../operador/entity/operador.entity';

@Injectable()
export class EquipoProgramaService {
  constructor(
    @InjectRepository(EquipoPrograma)
    private repository: Repository<EquipoPrograma>,
    @InjectRepository(FullInformacionEquipoProgramaView)
    private fullInformacionEquipoProgramaView: Repository<FullInformacionEquipoProgramaView>,
    @InjectRepository(InformacionEquipoProgramaView)
    private informacionEquipoProgramaView: Repository<InformacionEquipoProgramaView>,
    @Inject(forwardRef(() => EquipoService))
    private equipoService: EquipoService,
    private institucionProgramaService: InstitucionProgramaService,
  ) {}

  async create(
    // operador: Operador,
    id_equipo: number | Equipo,
    id_programa: number | Programa,
  ) {
    const equipo =
      typeof id_equipo === 'number'
        ? await this.equipoService.findInfoEquipoById(id_equipo)
        : id_equipo;
    const programa =
      typeof id_programa === 'number'
        ? await this.institucionProgramaService.findProgramaById(id_programa)
        : id_programa;

    return this.informacionEquipoPrograma(
      equipo.id_equipo,
      programa.id_programa,
    )
      .then((existeEquipoPrograma) => {
        if (existeEquipoPrograma)
          throw new ConflictException(
            'Este software ya fue asignado a este equipo.',
          );
        return this.informacionEquipoPrograma(equipo.id_equipo, 1);
      })
      .then(async (existeEquipoSinPrograma) => {
        if (existeEquipoSinPrograma)
          await this.repository.delete({
            id_equipo_programa: existeEquipoSinPrograma.id_equipo_programa,
          });
        return this.repository.save(
          this.repository.create({ equipo, programa }),
        );
      })
      .then((equipoPrograma) => ({
        message: `Se asignó el software: ${equipoPrograma.programa.programa}, al equipo con número de inventario: ${equipo.numero_inventario}.`,
        equipoPrograma,
      }));
  }

  async createCargaMasiva(equipo: Equipo, programa: Programa) {
    return this.informacionEquipoPrograma(equipo.id_equipo, 1)
      .then(async (existeEquipoSinPrograma) => {
        if (existeEquipoSinPrograma)
          await this.repository.delete({
            id_equipo_programa: existeEquipoSinPrograma.id_equipo_programa,
          });
        return this.repository.save(
          this.repository.create({ equipo, programa }),
        );
      })
      .then((equipoPrograma) => ({
        message: `Se asignó el software: ${equipoPrograma.programa.programa}, al equipo con número de inventario: ${equipo.numero_inventario}.`,
        equipoPrograma,
      }));
  }

  async delete(operador: Operador, id_equipo_programa: number) {
    const equipoPrograma = await this.findById(id_equipo_programa);

    if (
      operador.institucion.id_institucion !=
      equipoPrograma.equipo.carrito.modulo.institucion.id_institucion
    )
      throw new ForbiddenException(
        'No puedes eliminar el software de este equipo porque no pertenece a tu institución.',
      );
    return this.repository
      .count({ where: { equipo: equipoPrograma.equipo } })
      .then(async (n) => {
        if (n === 1) {
          if (equipoPrograma.programa.id_programa === 1)
            throw new ConflictException('No se puede eliminar esta opción.');
          await this.create(equipoPrograma.equipo, 1);
        }
        return this.repository.remove(equipoPrograma);
      })
      .then((_) => ({
        message: 'Se eliminó correctamente el software de este equipo.',
      }));
  }

  findById(id_equipo_programa: number) {
    return this.repository
      .findOne({
        join: {
          alias: 'ep',
          innerJoinAndSelect: {
            e: 'ep.equipo',
            p: 'ep.programa',
            c: 'e.carrito',
            m: 'c.modulo',
            i: 'm.institucion',
          },
        },
        where: { id_equipo_programa },
      })
      .then((equipoPrograma) => {
        if (!equipoPrograma)
          throw new ConflictException('No existe este id equipo software.');
        return equipoPrograma;
      });
  }

  findEquipoProgramaByEquipoPrograma(
    equipo: Equipo,
    programa: Programa,
    validarExistencia = true,
  ) {
    return this.repository
      .findOne({ where: { equipo, programa } })
      .then((equipoPrograma) => {
        if (validarExistencia && equipoPrograma)
          throw new ConflictException(
            'Este software ya fue asignado a este equipo.',
          );
        return equipoPrograma;
      });
  }

  findFullInfoEquipoProgramaAllByIdEquipo(id_equipo: number) {
    return this.fullInformacionEquipoProgramaView
      .find({
        where: { id_equipo },
      })
      .then((infoEquipoProgramas) => {
        const equipoProgramas: EquipoPrograma[] = [];

        for (let i = 0; i < infoEquipoProgramas.length; i++)
          equipoProgramas.push(
            this.repository.create({
              id_equipo_programa: infoEquipoProgramas[i].id_equipo_programa,
              programa: {
                id_programa: infoEquipoProgramas[i].id_programa,
                programa: infoEquipoProgramas[i].programa,
              },
            }),
          );
        return equipoProgramas;
      });
  }

  informacionEquipoPrograma(id_equipo: number, id_programa: number) {
    return this.informacionEquipoProgramaView.findOne({
      where: { id_equipo, id_programa },
    });
  }
}
